% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%read all NCC computations and output values
%For ERROR computation
clc
path_images = 'C:\Users\kumara4\ankur\Piston\code';
path_images = pwd;
filepath_images=strcat(path_images,'/images/24_February_2011/results/3t/vesselness/');%
filename_ncc_head = '3t_ch2_751-825_sm_deconv_15CLAHE_sigMAX_del15_NCC';
sample_string = '_smpl225';
filename_ncc_head = strcat(filepath_images, filename_ncc_head);
filename_vessl = strcat(filename_ncc_head, '_XDEFORM_vessl',sample_string,'.txt'); %deformed by Shape Context
filename_ncc = strcat(filename_ncc_head, '_NCCDeform_vessl.txt'); %deformed by NCC
filename_orig = strcat(filename_ncc_head, '_vessl.txt'); %undeformed

orig = dlmread(filename_orig);
vessl = dlmread(filename_vessl);
ncc = dlmread(filename_ncc);

%column 2 is NCC values, column 3 is correction/no-correction
indx_orig = orig(:,3)>0; %find all indices that need correction
indx_ncc = ncc(:,3)>0;
indx_vessl = vessl(:,3)>0;
orig_ncc = sum(orig(indx_orig,2)); %compute NCC for these indices
vessl_ncc = sum(vessl(indx_vessl,2));
ncc_ncc = sum(ncc(indx_ncc,2));

orig_corr = sum(orig(:,3));
vessl_corr = sum(vessl(:,3));
ncc_corr = sum(ncc(:,3));

disp('Number of slices NOT corrected')
[orig_corr, ncc_corr, vessl_corr]
disp('NCC values')
[orig_ncc, ncc_ncc, vessl_ncc]

