% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%compute Determinant of Hessian given Derivative Stk and sigma
%minpkdist: minpeakdistantce for finding peaks in DoH
%returns local_maxima of the determinant of the hessian
%channel2 contains RBC
function detHessStk = computeHessianDet(derStk, sigma, minpkdist, thresh)

depth = length(derStk);

% sz = size(derStk(1).Lxx(1,2).data);

sz = size(derStk(1).Lxx(1,1).data); %sz(1) is rows(height), sz(2) is cols(width)
%channel1, channel2
%detHess = struct('determinant',zeros(height,width));
detHessStk = repmat(struct('determinant',zeros(sz)),[1,depth]);
%detHL(x,y;t)=t^2 (Lxx*Lyy-Lxy^2)
for k=1:depth
    %compute all determinants for Hessian
    
    
    ch2_det = sigma.^2*(derStk(k).Lxx(1,1).data.*derStk(k).Lyy(1,1).data - derStk(k).Lxy(1,1).data.*derStk(k).Lxy(1,1).data);
    detHessStk(k).determinant = ch2_det;
    %compute local maxima at sigma-level, distance of 8
    
    
    %ch1_pks = zeros(size(ch1_reshape));
    
    %find local extrema, WHY NOT USE IMREGIONALMAX?????
    %[pks_ch1, loc_ch1] =  findpeaks(ch1_reshape,'minpeakdistance',minpkdist);
    %problem with findpeaks, if a neighborhood of values are below
    %threshold, a peak in that neighborhood will still be found
    ch2_reshape = reshape(ch2_det,1,[]);
    ch2_pks = zeros(size(ch2_reshape));
    %ABSOLUTE VALUE
    ch2_reshape = abs(ch2_reshape); %ABSOLUTE VALUE?
    ch2_reshape = max(ch2_reshape,thresh);
    [pks_ch2, loc_ch2] =  findpeaks(ch2_reshape,'minpeakdistance',minpkdist);

    ch2_pks(loc_ch2) = pks_ch2; %store local extrema
    ch2_max_det = reshape(ch2_pks, [sz(1) sz(2)]);

    detHessStk(k).max_determinant = ch2_max_det;
    fprintf('Computed determinant, scale=%d, Completed...(%d/%d)\n',sigma,k,depth);
end
