% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%compute MaxVesselness at multiple scales
%output: Vesselness, eigenvectors, lambda, scales

function maxVesslStk = computeMaxVesselness(eigHstk)

numscales = length(eigHstk.sigma);
maxVesslStk = eigHstk.data{1}; %maxVesselnessStack
depth = length(maxVesslStk);
sz = size(maxVesslStk(1).vesselness);
tic
%O(n^3), is there an efficient way? Optimization needed
for l=1:depth
    sigdata = zeros(sz);
    normfdata = zeros(sz);
    for i=1:sz(1) %rows, y-coordinate
        for j=1:sz(2) %cols, x-coordinate
            mtemp = zeros(1, numscales);
            for k=1:numscales
                tStk = eigHstk.data{k};
                %construct vesselness vector at all scales
                mtemp(k) = tStk(l).vesselness(i,j);
            end
            [vmax, imax] = max(mtemp); %find maximum
            sigdata(i,j) = eigHstk.sigma{imax}; %store the scale at which max was found
            maxVesslStk(l).vesselness(i,j) = vmax; %store the vesselness measure
            maxStk = eigHstk.data{imax}; %fetch maximum stack
            tmpnormf = maxStk.normf;
            normfdata(i,j) = tmpnormf(i,j); %write max-normfrobenius
            %store eigenvector and eigenvalues
            maxVesslStk(l).eigVec1(1,1).vector(i,j) = maxStk(l).eigVec1(1,1).vector(i,j);
            maxVesslStk(l).eigVec1(1,2).vector(i,j) = maxStk(l).eigVec1(1,2).vector(i,j);
            maxVesslStk(l).eigVec2(1,1).vector(i,j) = maxStk(l).eigVec2(1,1).vector(i,j);
            maxVesslStk(l).eigVec2(1,2).vector(i,j) = maxStk(l).eigVec2(1,2).vector(i,j);
            maxVesslStk(l).eig(1,1).lambdas(i,j) = maxStk(l).eig(1,1).lambdas(i,j);
            maxVesslStk(l).eig(1,2).lambdas(i,j) = maxStk(l).eig(1,2).lambdas(i,j);
        end
    end
    maxVesslStk(l).maxSigma = sigdata;
    maxVesslStk(l).normf = normfdata;
    fprintf('Computed maximum Vesselness Completed...(%d/%d)\n',l,depth);
end
toc