% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Motion Correction Framework for microscopy time-series images
%File: driver_main2.m
%Purpose: Select template file or build template file, non max supression,
    %and compute orientations

%Parameters -- see vesselnessDriver.m
%STK_FIRST_TIME - if TRUE, then find template, do not run motion-correction
%STK_FIRST_TIME - if FALSE, then run motion-correction, template exists in path
%low_range, high_range - range of frame numbers used to compute template, when STK_FIRST_TIME = 1



function maxVesslStk = driver_main2(filepath1,filename1,TemplateMATFilename,maxVesslStk,stk_vessl,stk_vessl_orig,stk_rbc_orig,STK_FIRST_TIME,low_range, high_range,epsillon_rigid,epsillon_nonrigid,number_samples,lambda_in,r)

if(STK_FIRST_TIME)
    j=1;
    sz = size(maxVesslStk(1).vesselness);
    vesslimgsum = zeros(sz);
    for i=low_range:high_range %copy the [low_range, high_range] vessel frames
        templateVesslStk(j) = maxVesslStk(i);
        vesslimgsum = vesslimgsum + 2.*templateVesslStk(j).vesselness;
        j=j+1; %increment
    end
    vesslimgsum = vesslimgsum ./ length(templateVesslStk); %average vesselness
    disp(['Template Vessel Stack length:' num2str(length(templateVesslStk))]);
    % nonmaxsup_driver
    templateVesslStk = nonmaxsup_driver(filepath1,filename1,templateVesslStk);
    % multipleOrientations
    templateVesslStk = multipleOrientations(filepath1,filename1,templateVesslStk);
    %build Template as .mat file
    maxTEMPLATEVesslStk.ORIENTATION_AVG = templateVesslStk(1).ORIENTATION_AVG;
    maxTEMPLATEVesslStk.ANGLES_AVG = templateVesslStk(1).ANGLES_AVG;
    maxTEMPLATEVesslStk.num_orientations = templateVesslStk(1).num_orientations; %get num orientations
    maxTEMPLATEVesslStk.backgroundOrientationValueAVG = templateVesslStk(1).backgroundOrientationValueAVG;
    maxTEMPLATEVesslStk.NMS_HYST_AVG = templateVesslStk(1).NMS_HYST_AVG;
    maxTEMPLATEVesslStk.vesselness_template = vesslimgsum; %template vesselness
    maxTEMPLATEVesslStk.low_range = low_range; %range used in computing template
    maxTEMPLATEVesslStk.high_range = high_range;
    savefilename = strcat(filepath1, TemplateMATFilename);
    %savefilename = strcat(filename1,'_TEMPLATE_maxVessl');
    TemplateMATFilename = savefilename;
    disp(['Saving template file.. ' savefilename]);
    save(savefilename,'maxTEMPLATEVesslStk','-v7.3');
    disp(['Saved template file.. ' TemplateMATFilename]);
    savefilename_tif = strcat(savefilename, '.tif');
    writeimg = uint8(255*(vesslimgsum./max(vesslimgsum(:))));
    imwrite(writeimg, savefilename_tif ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    disp('Completed building template file, ready for motion correction');
else
    STARTTIME = cputime;
    disp('executing all the programs...');
    disp(['Vessel Stack length:' num2str(length(maxVesslStk))]);
    template_fname = strcat(filepath1, TemplateMATFilename);
    disp(['Loading template file.. ' template_fname]);
    load(template_fname); %loads maxTEMPLATEVesslStk
    suffix_string = 'vessl';
    REGISTRATION_FLAG = 1;
    SC_FLAG = 0;
    [nccVesslStk, ncc_vec, ranges] = ncc_registration(filepath1, filename1, maxTEMPLATEVesslStk, maxVesslStk, REGISTRATION_FLAG, suffix_string, SC_FLAG);
    maxVesslStk(1).ncc_values = ncc_vec;
    maxVesslStk(1).ncc_flags = double(ranges);
    suffix_string = 'NCCDeform_vessl'; %find out how much was corrected using NCC
    REGISTRATION_FLAG = 0;
    [nccVesslStk, ncc_vec, ranges] = ncc_registration(filepath1, filename1, maxTEMPLATEVesslStk, nccVesslStk, REGISTRATION_FLAG, suffix_string, SC_FLAG);
    RUN_SC_CORR = 1;
    if(RUN_SC_CORR)

        % nonmaxsup_driver
        maxVesslStk = nonmaxsup_driver(filepath1,filename1,maxVesslStk);
        % multipleOrientations
        maxVesslStk = multipleOrientations(filepath1,filename1,maxVesslStk);
        % Build samples on the template and the time-series stack
        [template_info,slice_info] = motionCorr_GetSamplesDriver(filepath1,filename1,maxVesslStk,maxTEMPLATEVesslStk,number_samples);
        %perform motion-correction (registration) based on shape contexts
        maxVesslStk = motionCorr_driver(filepath1,filename1,maxVesslStk,stk_vessl,stk_vessl_orig,stk_rbc_orig,template_info,slice_info,maxTEMPLATEVesslStk,epsillon_rigid,epsillon_nonrigid,number_samples,lambda_in,r);
        suffix_string = 'XDEFORM_vessl';
        REGISTRATION_FLAG = 0;
        SC_FLAG = 1; %compute NCC deformed/corrected vesselness values
        %check how well motion corrections has performed
        [nccVesslStk, ncc_vec_sc, ranges_sc] = ncc_registration(filepath1, filename1, maxTEMPLATEVesslStk, maxVesslStk, REGISTRATION_FLAG, suffix_string, SC_FLAG);
        maxVesslStk(1).ncc_SC_values = ncc_vec_sc; 
        maxVesslStk(1).ncc_SC_flags = ranges_sc;
    end
    disp('TOTAL TIME for motion correction')
    TOTALTIME1 = cputime-STARTTIME
    disp('Completed motion correction')
end


