% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%motion correction GetSamples driver (using maxVesslStk)
%april 30 2011
%Input: post-NMS-hysteresis with multiple orientation images, maxVesslStk
%Issue: HOW DO YOU CHOOSE N and number_samples, this is a parameter
%output: deformation field

function [template_info,slice_info] = motionCorr_GetSamplesDriver(filepath1,filename1,maxVesslStk,maxTEMPLATEVesslStk,number_samples)
WRITE_FLAG = 1;
WRITE_FLAG_TEMPLATE = 0;
WRITE_FLAG_PERSLICE = 0;


outfn1 = strcat(filepath1, filename1);

lz = length(maxVesslStk); %length of maxVesslStk
ly = length(maxVesslStk(1).vesselness(:,1)); %num rows
lx = length(maxVesslStk(1).vesselness(1,:)); %num cols

% number_samples = 300; %important number to be selected



%get_samples_1(x,y,t,nsamp), nsamp = number_samples
%x,y: coordinates of boundary/orientation
%t: tangent, orientation-number

%FOR the vesslAVERAGE, template
orientation_template = maxTEMPLATEVesslStk.ORIENTATION_AVG;
angles_template = maxTEMPLATEVesslStk.ANGLES_AVG;
num_orients = maxTEMPLATEVesslStk.num_orientations; %get num orientations
bg_val_avg = maxTEMPLATEVesslStk.backgroundOrientationValueAVG;
all_avg_samples = zeros(ly,lx);
%TEMPLATE SAMPLING
tic
for ox=1:num_orients %0 is background pixel


    if(ox~=bg_val_avg)
        %for every orientation
        t_img = zeros(ly,lx);
        angs = zeros(ly,lx);
        indx = orientation_template == ox;
        
        t_img(indx) = orientation_template(indx); 
        angs(indx) = angles_template(indx); %select angles
        [r,c] = find(t_img==ox); %got all y,x coordinates        
        t = zeros(size(r));
        %build t (angle) vector based on [r,c]
        for tx_index=1:length(r)
            t(tx_index) = angs(r(tx_index),c(tx_index));
        end
        if(number_samples<length(r))
            nsamp = number_samples;
        else
            nsamp = floor(length(r)*0.8); %select 80% of the points
        end
        
        if(nsamp>0)
            [ox nsamp length(r)]


            [xc, yc, tc] = get_samples_1(c,r,t,nsamp); %x,y,t,ind0 NEED TO STORE
            Y = [xc yc];
            template_info(ox).samples.coords = Y;
            template_info(ox).samples.angles = tc;
            
            %write to a file
            if(WRITE_FLAG_TEMPLATE && nsamp>0)
                t_img = zeros(ly,lx); %reset to zero
                for ix=1:length(xc)
                    t_img(yc(ix),xc(ix)) = ox; %individual marking
                    all_avg_samples(yc(ix),xc(ix)) = ox; %mark on the ALL image
                end
                %write to output file per orientation
                outfname1 = strcat(outfn1, '_NMShyst','_orient',num2str(num_orients),'_o-',num2str(ox),'smpl',num2str(number_samples),'_vesslAVERAGE.tif');
                t_img = uint8(255*(t_img./max(t_img(:)))); 
                imwrite(t_img, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
                %write output lz times
                outfname2 = strcat(outfn1, '_NMShyst','_orient',num2str(num_orients),'_o-',num2str(ox),'smpl',num2str(number_samples),'_vesslAVERAGE1-',num2str(lz),'.tif');
                for jx=1:lz
                    imwrite(t_img, outfname2 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
                end
            end
        end
    end
end
%write all_avg_samples to output file
if(WRITE_FLAG)
    outfname1 = strcat(outfn1, '_NMShyst','_orient',num2str(num_orients),'_o-ALL','smpl',num2str(number_samples),'_vesslAVERAGE.tif');
    all_avg_samples = uint8(255*(all_avg_samples./max(all_avg_samples(:)))); 
    imwrite(all_avg_samples, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end
toc
disp('RUNNING PER SLICE SAMPLING...')
% % number_samples = 300; %important number to be selected, should be double for target?
tic
%PER SLICE SAMPLING
kx=1;
for ox=1:num_orients %0 is background pixel
    %output file per orientation

    bg_val = maxVesslStk(kx).backgroundOrientationValue;

    if(ox~=bg_val)
        for k=1:lz
            

            %for every orientation
            dfnum=0;
            t_img = zeros(ly,lx);
            angs = zeros(ly,lx);
            o_t = maxVesslStk(k).orientations;
            indx = o_t == ox;
            t_img(indx) = o_t(indx); %select orientations
            angle_vec = maxVesslStk(k).angles;
            angs(indx) = angle_vec(indx); %select angles
            [r,c] = find(t_img==ox); %got all y,x coordinates
            t = zeros(size(r));
            %build t (angle) vector based on [r,c]
            for tx_index=1:length(r)
                t(tx_index) = angs(r(tx_index),c(tx_index));
            end
            if(~isempty(template_info(ox).samples))
            nsamp = length(template_info(ox).samples.coords);

            if(nsamp<length(r))
                nsamp = number_samples; %good flag
            else
                %length(r)<number_samples .. issue
                %repeat the difference
                if(~isempty(r))
                    dfnum = nsamp-length(r)+5;
                    valr = r(length(r)); valc = c(length(c)); valt = t(length(t));
                    r = [r;repmat(valr,dfnum,1)];
                    c = [c;repmat(valc,dfnum,1)];
                    t = [t;repmat(valt,dfnum,1)];
                end
            end
            
            if(dfnum>0) %store bad slices
                slice_info(ox).samples(k).flag = 1;
            else
                slice_info(ox).samples(k).flag = 0;
            end
            

            if(nsamp>0)
                nsamp = length(template_info(ox).samples.coords); %keep same length as template

                [k nsamp length(r) dfnum ox slice_info(ox).samples(k).flag]

                [xc, yc, tc] = get_samples_1(c,r,t,nsamp); %x,y,t, NEED TO STORE
                
                Y=[xc yc];
                slice_info(ox).samples(k).coords = Y;
                slice_info(ox).samples(k).angles = tc;
                if(WRITE_FLAG_PERSLICE)
                    outfname1 = strcat(outfn1, '_NMShyst','_orient',num2str(num_orients),'_o-',num2str(ox),'smpl',num2str(number_samples),'_vessl.tif');
                    t_img = zeros(ly,lx); %reset to zero
                    %write to a file
                    for ix=1:length(xc)
                        t_img(yc(ix),xc(ix)) = ox; %individual marking
                    end
                    %write to output file
                    t_img = uint8(255*(t_img./max(t_img(:)))); 
                    imwrite(t_img, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
                end
            end
            end
        end
        [ox nsamp length(r)]
    end
end
toc
savefilename = strcat(filename1,'_maxVessl_SAMPLES',num2str(number_samples));
disp(['Saving samples...', savefilename])
save(savefilename,'template_info','slice_info','-v7.3');

