% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%File: motionCorr_driver.m
%Purpose: Run motion-correction. TPS warps after Cost, Hungarian, Bookstein

%input: samples (slice_info) and template (template_info)

function maxVesslStk = motionCorr_driver(filepath1,filename1,maxVesslStk,stk_vessl,stk_vessl_orig,stk_rbc_orig,template_info,slice_info,maxTEMPLATEVesslStk,epsillon_rigid,epsillon_nonrigid,number_samples,lambda_in,r)

gold_slice_info = slice_info;
gold_template_info = template_info;

%Frames to Correct
ncc_flags = double(maxVesslStk(1).ncc_flags); %motion_correction=1, no_motion_correction=0
len_ncc = length(ncc_flags); %total number of slices
ncc_ranges_all = (1:len_ncc)'.*ncc_flags; %fetch slice numbers
ncc_ranges = ncc_ranges_all(ncc_ranges_all~=0); %got all indices
totalranges = length(ncc_ranges); %number of motion corrections to perform

%FLAGS
ITERATIVE_RIGID_FLAG = 0; %do not set, bad results, used for testing methods
NONRIGID_FLAG_RUN = 1; %important
REDUCE_DOF_FLAG = 0; %do not set, bad results, used for testing methods
WRITE_FLAG = 1;
SAVE_MAXVESSLSTK = 0;
n_good_hungarian_vec = [];
DISPLAY_FLAG = 0;
BOOKSTEIN_FLAG = 1; %use Bookstein's TPS, else use ROHR
INCORRECT_HUNGARIAN_FLAG = 1;
INVALID_ALL_ORIENTATIONS_FLAG = 0;
% REDUCE_DOF_FLAG = 1; %reduce degrees of freedom for bookstein
% NONRIGID_FLAG_RUN = 1; %enter non-rigid code
RIGID_RUN_FLAG = 1;
NONRIGID_FLAG_RUN_GOLD = NONRIGID_FLAG_RUN;
RIGID_RUN_FLAG_GOLD = RIGID_RUN_FLAG;
NONRIGID_FLAG = 1; %'ss' in previous files, end the iterations for non-rigid
epsillon_nonrigid_gold = epsillon_nonrigid;
max_thresh_dof = 2*epsillon_nonrigid;

%parameters for execution of vessel context using demo_2
affine_start_flag = 1;
polarity_flag = 1; %orientations
distance_cost_flag = 1; %distance cost flag
eps_dum = 0.25;
ndum_frac = 0.25;
mean_dist_global = [];
tan_eps = 1.0;
% beta_init = 1;
% lambda_in=100;
% r=0.4;
beta_init = lambda_in;
step = 32;
% r = 1;
w = 4;
sf = 2.5;



%Parameters for vessel context settings
n_iter_rigid = 6;
n_iter = 6;
nbins_theta = 12;
r_inner = 1/8;
r_outer = 4;
nbins_r = 11;
lambda_dist_weight = 0.1;
ori_weight = 0.1; 
%step = 16; %for averaging deformation field, Degrees of Fredom MULTIPLE of DIMENSIONS
% logspace(log10(r_inner),log10(r_outer),nbins_r)

lz = length(maxVesslStk); %length of maxVesslStk
ly = length(maxVesslStk(1).vesselness(:,1)); %num rows
lx = length(maxVesslStk(1).vesselness(1,:)); %num cols

num_divs = length(template_info);  %number of orientations
[N1,N2] = size(maxVesslStk(1).vesselness); %size

fig_index = 1;
f_index = 0;

%a deformable grid
sqsize = 4;
yl = 1:sqsize:ly; %rows
xl = 1:sqsize:lx; %cols
def_grid = zeros(ly,lx);
def_grid(yl,:) = 255;
def_grid(:,xl) = 255;

orientation_index = []; %valid orientation indices


if(REDUCE_DOF_FLAG)
    n_iter = 1;
    affine_start_flag = 0; 
    r = 0.93;
    %step = 32;
    max_thresh_dof = epsillon_nonrigid;
end
% r = 0.93;

for ox=1:num_divs
    template_orient_samples = template_info(ox).samples;
    
    if(~isempty(template_orient_samples)) %not empty
        orientation_index = [orientation_index;ox]; %store orientation index value
        angle_struct_t1.orient(ox).data = [];
        angle_struct_t2.orient(ox).data = [];
        X_struct.orient(ox).data = [];
        Y_struct.orient(ox).data = [];
    end
end

%register Template(symbolized by '2') to Slice (symbolized by '1')

bad_slices = [];
Ybag = []; Xbag = []; t2bag = []; t1bag = [];
fre_square_mag_vec = zeros(lz,1);

%%ERROR CALCULATION residual storage

for indx_corr=1:totalranges 
	kx = ncc_ranges(indx_corr);%for each slice to be motion corrected
	disp(['Start Correction of slice number.. [', int2str(kx), ' / ', int2str(totalranges),']']);
    RIGID_RUN_FLAG = RIGID_RUN_FLAG_GOLD; %reset
    fre_sq_vec = zeros(1,n_iter_rigid); %reset
    fre_sq_max = realmax; %reset
    max_HX = eye(3); %reset
    max_Xk = []; %reset
    max_Y = []; %reset
    counter = 1; %reset
    TRX = eye(3); %reset
    HX = eye(3); %reset
    TtX = [0;0]; %reset
    while(RIGID_RUN_FLAG)
        n_good_vector = zeros(size(orientation_index));
        tic
        %reset all bags for next slice
        Ybag = []; Xbag = []; t2bag = []; t1bag = [];
        for or_index=1:length(orientation_index) %for each orientation
            ox = orientation_index(or_index); %get orientation index
            %build bag of points for slice_info and template_info
            disp(['iter=' int2str(counter)])
            if(slice_info(ox).samples(kx).flag==0) %is this orientation flagged?

                
                if(counter>1) %iterative rigid transformation update ALL of the above variables
                    disp('using X_struct and Y_struct, counter>1')
                    X = X_struct.orient(or_index).data; %should be transformed
                    Y = Y_struct.orient(or_index).data;
                    nsamp_slice = length(X_struct.orient(or_index).data);
                    nsamp_template = length(Y_struct.orient(or_index).data);
                    t1 = angle_struct_t1.orient(or_index).data;
                    t2 = angle_struct_t2.orient(or_index).data;
                    
                else
                    %iteration = 0
                    disp('running defaults X Y, counter<=1')
                    nsamp_template = length(template_info(ox).samples.coords); %number of samples for template
                    nsamp_slice = length(slice_info(ox).samples(1).coords); %number of samples for slices (targets)
                    golden_nsamp = nsamp_slice;
                    %run Hungarian per orientation then concat assignments
                    Y = template_info(ox).samples.coords; %template samples
                    t2 =  template_info(ox).samples.angles;
                    X = slice_info(ox).samples(kx).coords; %slice samples
                    t1 =  slice_info(ox).samples(kx).angles;
                    %all have same sizes
                    X_struct.orient(or_index).data = X; %store to update with RX and tX
                    Y_struct.orient(or_index).data = Y; %store
                    angle_struct_t1.orient(or_index).data = t1; %store
                    angle_struct_t2.orient(or_index).data = t2; %store
                end

                x2 = Y(:,1); y2 = Y(:,2); %template
                x1 = X(:,1); y1 = X(:,2); %slice
                if (DISPLAY_FLAG)
                    figure(1) %figure number proportional to orientation
                    subplot(2,1,1)
                    plot(Y(:,1),Y(:,2),'b+')
                    hold on
                    quiver(Y(:,1),Y(:,2),cos(t2),sin(t2),0.5,'b')
                    hold off
                    axis('ij');axis([1 N2 1 N1])
                    title([int2str(length(x2)) ' samples template'])
                    subplot(2,1,2)
                    plot(X(:,1),X(:,2),'ro')
                    hold on
                    quiver(X(:,1),X(:,2),cos(t1),sin(t1),0.5,'r.')
                    hold off
                    axis('ij');axis([1 N2 1 N1])
                    title([int2str(length(x1)) ' samples slice'])
                    drawnow	
                end
                if(DISPLAY_FLAG)
                    [x,y] = meshgrid(linspace(1,N2,36),linspace(1,N1,36));
                    x=x(:); y=y(:); M=length(x);
                end

                %COMPUTE CORRESPONDENCES for this orientation
                Xk = X;
                tk = t1;
                k = 1;
                ndum = round(ndum_frac*nsamp_slice); %limit by template, slice_info should have outliers
                out_vec_1 = zeros(1, nsamp_slice);
                out_vec_2 = zeros(1, nsamp_template);

                disp('iter = RIGID_BODY_REGISTRATION')
                disp(['computing shape contexts for orientation=' int2str(ox) '(deformed)[slice_info ' int2str(kx) '] model...'])
                [BH1,mean_dist_1]=sc_compute(Xk',zeros(1,nsamp_slice),mean_dist_global,nbins_theta,nbins_r,r_inner,r_outer,out_vec_1);
                disp('computing shape contexts for (template)[template_info] target...')
                [BH2,mean_dist_2]=sc_compute(Y',zeros(1,nsamp_template),mean_dist_global,nbins_theta,nbins_r,r_inner,r_outer,out_vec_2);
                disp('done.')

                if (affine_start_flag)
                    if (counter == 1)
                        %use huge regularization to get affine behavior
                        lambda_o = 1000;
    
                    else
                        lambda_o = beta_init*r^(k-2);

                    end
                else
                    lambda_o = beta_init*r^(k-1);
                end
                beta_k = (mean_dist_2^2)*lambda_o;
                disp('computing histogram costs, chi-squared, of shape contexts [RIGID]')
                costmat_shape = hist_cost_2(BH1,BH2);

                theta_diff = repmat(tk,1,nsamp_template)-repmat(t2',nsamp_slice,1);
                if(polarity_flag)
                    %use edge polarity based on angles
                    costmat_theta = 0.5*(1-cos(theta_diff));
                else
                    costmat_theta = 0.5*(1-cos(2*theta_diff));
                end
                costmat = (1-ori_weight)*costmat_shape+ori_weight*costmat_theta; %original
                
                %------start distance function
                if(distance_cost_flag)
                    if (counter==1)
                        dk = zeros(size(tk));
                        d2 = zeros(size(t2));
                        cost_dist_diff = repmat(dk(:,1), 1, nsamp_template)-repmat(d2(:,1)', nsamp_slice,1);
                        epsillon_nonrigid = epsillon_nonrigid_gold;
                    else
                        %compute distance for k>1 iteration
                        disp('computing distance cost matrix')
                        tmp = zeros(1,2); %add dummy
                        dumXk = [tmp;Xk];
                        dumY = [tmp;Y];
                        j=sqrt(-1);
                        dXk = dumXk(:,1) + j*dumXk(:,2); %make a+bi
                        dY = dumY(:,1) + j*dumY(:,2); 
                        diffXk = diff(dXk); %find differences between neighboring points, size should be 1 less
                        diffY = diff(dY); %find differernces between neighboring points, size should be 1 less
                        dk = abs(diffXk); %magnitude of difference between neighboring points
                        d2 = abs(diffY);  %magnitude of difference between neighboring points
                        cost_dist_diff = repmat(dk, 1, nsamp_template)-repmat(d2', nsamp_slice,1);
                        A = cost_dist_diff;
                        NA = (A-min(A(:))+realmin)./ (max(A(:))-min(A(:)));
                        cost_dist_diff = NA; %normalize dist cost
                        disp('done.')

                    end
                else
                        dk = zeros(size(tk));
                        d2 = zeros(size(t2));
                        epsillon_nonrigid = epsillon_nonrigid_gold;
                        cost_dist_diff = repmat(dk(:,1), 1, nsamp_template)-repmat(d2(:,1)', nsamp_slice,1);
                end
                %Cost Matrix
                costmat = (1-ori_weight-lambda_dist_weight)*costmat_shape + ori_weight*costmat_theta + lambda_dist_weight*cost_dist_diff;
                nptsd = nsamp_slice + ndum;
                
                %------end distance function
    
                nptsd = nsamp_slice + ndum;
                costmat2 = eps_dum*ones(nptsd, nptsd);
                costmat2(1:nsamp_slice,1:nsamp_template) = costmat;
                run_costmat = costmat;
                %SOLVE ASSIGNMENT PROBLEM - hungarian O(n^3)
                disp('hungarian... RIGID')
                cvec = hungarian(costmat2);
                %update outlier indicator vectors
                [a,cvec2] = sort(cvec); %sort according to optimal assignments
                out_vec_1 = cvec2(1:nsamp_slice)>nsamp_template; %logical markings
                out_vec_2 = cvec(1:nsamp_template)>nsamp_slice;

                X2 = NaN*ones(nptsd,2);
                X2(1:nsamp_slice,:) = Xk;
                X2 = X2(cvec,:);
                X2b = NaN*ones(nptsd,2);
                X2b(1:nsamp_slice,:) = X;
                X2b = X2b(cvec,:);
                Y2 = NaN*ones(nptsd,2);
                Y2(1:nsamp_template,:) = Y;

                %extract coordinates of non-dummy correspondences and
                %use them to estimate transformations
                ind_good = find(~isnan(X2b(1:nsamp_slice,1)));
                n_good = length(ind_good);
                X3b = X2b(ind_good, :);
                Y3 = Y2(ind_good, :);
                t1new = tk(ind_good,:);
                t2new = t2(ind_good,:);
                %remove incorrect/outlier/impossible Hungarian matches
                X4b=[]; Y4=[];
                if(INCORRECT_HUNGARIAN_FLAG)
                    X4b = X2b(ind_good,:); Y4 = Y2(ind_good,:);
                    X4b_1 = X4b(:,1); Y4_1 = Y4(:,1); %x-coord
                    X4b_2 = X4b(:,2); Y4_2 = Y4(:,2); %y-coord
                    %find distances
                    dvec = sqrt((X4b_1-Y4_1).^2 + (X4b_2-Y4_2).^2);
                    dvec_pbr = dvec;
                    epsillon_rigidtmp = mean(dvec)+std(dvec);
                     
                    if(counter==1)
                        dvec_indx = dvec <= epsillon_rigid; %comparison

                        eps_rig_to_use = epsillon_rigid;
                    else


                        dvec_indx = dvec <= epsillon_rigid; %comparison
                        eps_rig_to_use = epsillon_rigid;
                        
                    end
                    
                    X3b = X3b(dvec_indx,:);
                    Y3 = Y3(dvec_indx,:);
                    n_good = sum(dvec_indx);
                    n_good_pbr = n_good;
                    t1new = t1new(dvec_indx,:);
                    t2new = t2new(dvec_indx,:);
                    disp(['DistanceVec RIGID mode, Threshold= ' num2str(mean(dvec)+std(dvec)) ', usedThreshold= ' num2str(eps_rig_to_use) ', n_good= ' int2str(n_good)])
                end
                n_good_vector(or_index) = n_good_pbr;
                
                
                Ybag = [Ybag; Y3];
                Xbag = [Xbag; X3b];
                t1bag = [t1bag; t1new];
                t2bag = [t2bag; t2new];
                
                disp(['VALID orienation completed slice_info ' num2str(kx) ' orientation= ' num2str(ox) ' flag=' num2str(slice_info(ox).samples(kx).flag)]);
            else    
                disp(['INVALID orienation slice_info ' num2str(kx) ' orientation= ' num2str(ox) ' flag=' num2str(slice_info(ox).samples(kx).flag)]);
                %go to different orientation
            end
            
        end
        %all orientations have been considered for matching using Hungarian
        disp('rigid body registration, set-up..')
        %transform xyrot to xy, from example
        X3b = Xbag;
        Y3 = Ybag;
        rms.noncorrRMS(kx).Xbag = X3b;
        rms.noncorrRMS(kx).Ybag = Y3;
        X5b = X3b'; %xyrot
        Y5 = Y3'; %xy
        if(isempty(Y5))
            disp(['INVALID FLAGGED SLICE IN ALL ORIENTATIONS, copying initial coords slice number=' num2str(kx)]);
            X5bag = []; Y5bag = [];
            for or_index=1:length(orientation_index)
                if(slice_info(ox).samples(kx).flag==1) %make sure all the flags=1
                    X5b = slice_info(ox).samples(kx).coords;
                    Y5 = X5b;
                    Y5bag = [Y5bag; Y5];
                    X5bag = [X5bag; X5b];
                end
            end
            X5b = X5bag';
            Y5 = Y5bag';
            INVALID_ALL_ORIENTATIONS_FLAG = 1;
            NONRIGID_FLAG_RUN = 0; %don't run nonrigid
        else
            NONRIGID_FLAG_RUN = NONRIGID_FLAG_RUN_GOLD; %whatever the user wanted
        end
        A_bar = mean(Y5,2); %xy
        B_bar = mean(X5b,2); %xyrot
        disp('sizes of Y5 A_bar repmat')
        size(Y5)
        size(A_bar)
        size(repmat(A_bar,1,length(Y5(1,:))));
        A_tilda = Y5-repmat(A_bar,1,length(Y5(1,:)));
        B_tilda = X5b-repmat(B_bar,1,length(X5b(1,:)));
        disp('rigid body registration, covariance..')
        cov_mat = B_tilda*A_tilda';
        disp('rigid body registration, SVD..')
        [u,s,v] = svd(cov_mat);
        RX = v*diag([1,det(v*u)])*u'
        tX =  A_bar - RX*B_bar


        TRX = [RX, tX];
        TRX = [TRX; 0 0 1]


        HX = TRX*HX

        
        %reg = RX*X5b + repmat(tX,1,length(X5b(1,:)));
        RX = HX(1:2, 1:2);
        tX = HX(:,3); tX = tX(1:2);
        reg = RX*X5b + repmat(tX,1,length(X5b(1,:)));
        X3b = reg';
        disp('rigid body registration, done..')
        toc
        fre_squared = (1/length(Y5(1,:)))*sum(abs(reg-Y5).^2,2)
        fre_sq_vec(counter) = sqrt(fre_squared(1).^2 + fre_squared(2).^2);
        Xk = X3b; %transformation for the entire bag
        Y = Y5';
        Y3 = Y;
        
        if(fre_squared < fre_sq_max)
            fre_sq_max = fre_squared;
            max_HX = HX;
            max_Xk = Xk;
            max_Y = Y;
            max_Y3 = Y3;
            max_indx_rigid = counter;
            max_t1bag = t1bag;
            max_t2bag = t2bag;
            max_n_good_vector = n_good_vector;
        end
        
        disp('n_good_vector from rigid body registration')
        n_good_vector'
        

        if(ITERATIVE_RIGID_FLAG)
            if (counter == n_iter_rigid)
                RIGID_RUN_FLAG = 0; %end loop
            else
                if(~INVALID_ALL_ORIENTATIONS_FLAG)
                    disp('Rigid transformation per orientation, iterative rigid..');
                    
                    for tr_index=1:length(orientation_index)
                       tox = orientation_index(tr_index); %get orientation index
                       %build bag of points for slice_info and template_info
                       if(slice_info(tox).samples(kx).flag==0)
                           %rigid transformation of all the points in each
                           %orientation, tr_index
                           
                           %size: X_struct = Y_struct = angle_struct all
                           %the time, everytime
                           
                           X7b = X_struct.orient(tr_index).data; %retreive the good slice points used in rigid registration
                           Y7 = Y_struct.orient(tr_index).data; %retreive the corresponding template points

                           X7b = X7b';
                           BX = HX;
                           disp('using HX for transformation')
                           RX = BX(1:2, 1:2);
                           tX = BX(:,3); tX = tX(1:2);
                           

						   reg = RX*X7b + repmat(tX,1,length(X7b(1,:)));
                           X7b = reg';
                           %update transformation per orientation
                           X_struct.orient(tr_index).data = X7b; %update for the an iteration of rigid registration
                           n_good_vector(tr_index) = length(X7b(:,1));
                           
                            if(DISPLAY_FLAG && tox==7)
                                fig_index = fig_index + 1;
                                figure(fig_index)

                                % show the correspondences between the
                                % untransformed images for all orientations

                                hold on
                                plot(X7b(:,1),X7b(:,2),'gs',Y7(:,1),Y7(:,2),'md');
                                plot([X7b(:,1) Y7(:,1)]',[X7b(:,2) Y7(:,2)]','k-')
                                hold off
                                axis('ij')
                                title([int2str(length(X7b(:,1))) ' correspondences (unwarped X/slice)' ', iter=' int2str(counter) ', ox=' int2str(tox)])
                                axis([1 N2 1 N1])
                                drawnow
                           end
                       
                       end
                    end
                    disp('iterating counter..')
                    counter = counter+1;
                else
                    disp('INVALID_ALL_ORIENTATIONS_FLAG=1 all orientations are invalid.. there is no R, tX, setting counter=n_iter_rigid')
                    counter = n_iter_rigid; %end the loop
                end
            end
        else
            RIGID_RUN_FLAG = 0;
        end
        disp(['starting iter=' int2str(counter)])
    
    end %end while loop
    %---------------end rigid
    disp('---------------------- END RIGID---')
    disp('FINAL n_good_vector from rigid body registration')
    n_good_vector = max_n_good_vector'
    %DISPLAY_FLAG = 0;
    disp('FINAL FRE_SQUARE_vector from rigid body registration')
    fre_sq_vec
    max_indx_rigid
    disp('total/final transformation according to min-FRE..')
    max_HX
    disp('total/final transformation.. to be used ')
    HX = TRX
    NONRIGID_FLAG_RUN = 1;
    if(DISPLAY_FLAG) %works only for ONE SLICE
        for tr_index=3:3
           tox = orientation_index(tr_index); %get orientation index
           %build bag of points for slice_info and template_info
           if(slice_info(tox).samples(kx).flag==0)
                
               X7b = X_struct.orient(tr_index).data;
               Y7 = Y_struct.orient(tr_index).data;
               
                fig_index = fig_index + 1;
                figure(fig_index)

                % show the correspondences between the
                % untransformed images for all orientations
        
                hold on
                plot(X7b(:,1),X7b(:,2),'gs',Y7(:,1),Y7(:,2),'md');
                plot([X7b(:,1) Y7(:,1)]',[X7b(:,2) Y7(:,2)]','k-')
                hold off
                axis('ij')
                title([int2str(length(X7b(:,1))) ' correspondences (unwarped X/slice)' ', iter=' int2str(counter) ', ox=' int2str(tox)])
                axis([1 N2 1 N1])
                drawnow
           end
        end
    end
    
   if(DISPLAY_FLAG)
        fig_index = fig_index + 1;
        X7b=max_Xk;
        Y7=max_Y;
        figure(fig_index)

        % show the correspondences between the
        % untransformed images for all orientations

        hold on
        plot(X7b(:,1),X7b(:,2),'gs',Y7(:,1),Y7(:,2),'md');
        plot([X7b(:,1) Y7(:,1)]',[X7b(:,2) Y7(:,2)]','k-')
        hold off
        axis('ij')
        title([int2str(length(X7b(:,1))) ' correspondences (unwarped X/slice)' ', iter=' int2str(counter) ', ox=' int2str(tox)])
        axis([1 N2 1 N1])
        drawnow
   end
    

    newX3bn = Xk;
    newY3n = Y;
    %code for non-rigid registration for a slice
    disp(['INVALID_ALL_ORIENTATIONS_FLAG= ' int2str(INVALID_ALL_ORIENTATIONS_FLAG)]);
    if(NONRIGID_FLAG_RUN && ~INVALID_ALL_ORIENTATIONS_FLAG)
        tic
        disp('setting up for non-rigid registration...')
        INVALID_ALL_ORIENTATIONS_FLAG = 0;
        %UPDATE ALL PARAMETERS
        NONRIGID_FLAG = 1;
        Xnew = max_Xk; %contains all new Xbag slice
        Ynew = max_Y; %contains all new Ybag template


        HX = TRX;

        Y0 = Ynew;
        Xn = Xnew;
        Yn = Ynew;
        X = Xnew;
        %sbvec = Yn-Xn;
        %rms.RigidcorrRMS(kx).data = sbvec;
        rms.RigidcorrRMS(kx).Xbag = Xn;
        rms.RigidcorrRMS(kx).Ybag = Yn;
        nsamp_slice_0 = length(Xnew(:,1));
        nsamp_template_0 = length(Ynew(:,1));
        t1new = max_t1bag; %contains all t1bag - slice angles
        t2new = max_t2bag; %contains all t2bag - template angles


        tk = t1new;
        t1 = tk;
        t2 = t2new;
        t2N = t2;
        tkN = tk;
        x2 = Y(:,1);
        y2 = Y(:,2);
        nsamp_slice = length(Xnew(:,1));
        nsamp_template = length(Y(:,1));
        k = 1;
        ndum = round(ndum_frac*nsamp_slice); %limit by template, slice_info should have outliers
        
        
        %remember n_good_vector stores how many n_good per orientation
        while NONRIGID_FLAG %run n_iter times

            %execute Hungarian per orientation, then concatenate for TPS
            start_n = 0;
            run_n_good = 0;
            %run_costmat = [];
            X3bag = [];
            Y3bag = [];
            %dvecbag=[];
            tknbag = [];
            t2nbag = [];
            sc_cost = 0;
			n_good_hungarian_vec = [];
            for nx=1:length(n_good_vector) %get assignments per orientation

                n_good = n_good_vector(nx); %index access: start_n+1:start_n+n_good, start_n=n_good
                if(n_good>0)
                    disp(['iter= ' int2str(k)])
                    Xk = Xn(start_n+1:start_n+n_good,:);
                    Y = Yn(start_n+1:start_n+n_good,:);
                    tk = tkN(start_n+1:start_n+n_good,:);
                    t2 = t2N(start_n+1:start_n+n_good,:);
                    X_0 = X(start_n+1:start_n+n_good,:);
                    Y_0 = Y0(start_n+1:start_n+n_good,:);
                    %dvec_vec = start_n+1:(start_n+n_good);
                    
                    nsamp_slice = length(X_0(:,1)); %update nsamp_slice for an orientation
                    nsamp_template = length(Y_0(:,1)); %update nsamp_template for an orientation
                    out_vec_1 = zeros(1, nsamp_slice);
                    out_vec_2 = zeros(1, nsamp_template);
                    
                    if (affine_start_flag)
                        if (k == 1)
                            %use huge regularization to get affine behavior
                            lambda_o = 1000;
                        else
                            lambda_o = beta_init*r^(k-2);
                        end
                    else
                        lambda_o = beta_init*r^(k-1);
                    end
                    beta_k = (mean_dist_2^2)*lambda_o;
                    disp('iter = NONRIGID_BODY_REGISTRATION')
                    %insert 
                    %if(~ITERATIVE_RIGID_FLAG || (ITERATIVE_RIGID_FLAG && k==1))
                        %disp('iter = NONRIGID_BODY_REGISTRATION[NBR], ITERATIVE_RIGID_FLAG = FALSE')
                        disp(['computing shape contexts for orientation=' int2str(nx) '(deformed)[slice_info ' int2str(kx) '] model...' ' indices [' int2str(start_n+1) ',' int2str(start_n+n_good) ']'])
                        [BH1,mean_dist_1]=sc_compute(Xk',zeros(1,nsamp_slice),mean_dist_global,nbins_theta,nbins_r,r_inner,r_outer,out_vec_1);
                        disp('computing shape contexts for (template)[template_info] target...')
                        [BH2,mean_dist_2]=sc_compute(Y',zeros(1,nsamp_template),mean_dist_global,nbins_theta,nbins_r,r_inner,r_outer,out_vec_2);
                        disp('done.')
                        start_n = start_n+n_good; %update start_n
                        %maxVesslStk(kx).deform.sc_hist = BH1; %the last updated shape context histogram during registration
                        
                        %compute COST for shape, histogram cost, chi^2?
                        disp('computing histogram costs of shape contexts')
                        costmat_shape = hist_cost_2(BH1,BH2);

                        theta_diff = repmat(tk,1,nsamp_template)-repmat(t2',nsamp_slice,1);
                        if(polarity_flag)
                            %use edge polarity based on angles
                            costmat_theta = 0.5*(1-cos(theta_diff));
                        else
                            costmat_theta = 0.5*(1-cos(2*theta_diff));
                        end

                        %DISTANCE COMPUTATION
                        %distance_cost_matrix, k==1 initialization
                        if(distance_cost_flag)
                            if (k==1)
                                dk = zeros(size(tk));
                                d2 = zeros(size(t2));
                                cost_dist_diff = repmat(dk(:,1), 1, nsamp_template)-repmat(d2(:,1)', nsamp_slice,1);
                                epsillon_nonrigid = epsillon_nonrigid_gold;
                            else
                                %compute distance for k>1 iteration
                                disp('computing distance cost matrix')
                                tmp = zeros(1,2); %add dummy
                                dumXk = [tmp;Xk];
                                dumY = [tmp;Y];
                                j=sqrt(-1);
                                dXk = dumXk(:,1) + j*dumXk(:,2); %make a+bi
                                dY = dumY(:,1) + j*dumY(:,2); 
                                diffXk = diff(dXk); %find differences between neighboring points, size should be 1 less
                                diffY = diff(dY); %find differernces between neighboring points, size should be 1 less
                                dk = abs(diffXk); %magnitude of difference between neighboring points
                                d2 = abs(diffY);  %magnitude of difference between neighboring points
                                cost_dist_diff = repmat(dk, 1, nsamp_template)-repmat(d2', nsamp_slice,1);
                                A = cost_dist_diff;
                                NA = (A-min(A(:))+realmin)./ (max(A(:))-min(A(:)));
                                cost_dist_diff = NA; %normalize dist cost
                                disp('done.')
    
                            end
                        else
                            dk = zeros(size(tk));
                            d2 = zeros(size(t2));
                            epsillon_nonrigid = epsillon_nonrigid_gold;
                            cost_dist_diff = repmat(dk(:,1), 1, nsamp_template)-repmat(d2(:,1)', nsamp_slice,1);
                        end
                        %Cost Matrix
                        costmat = (1-ori_weight-lambda_dist_weight)*costmat_shape + ori_weight*costmat_theta + lambda_dist_weight*cost_dist_diff;
                        nptsd = nsamp_slice + ndum;
                        costmat2 = eps_dum*ones(nptsd, nptsd);
                        costmat2(1:nsamp_slice,1:nsamp_template) = costmat;
                        run_costmat = costmat;

                        %hungarian: O(n^3)
                        disp('hungarian... NONRIGID')
                        cvec = hungarian(costmat2);
                        [a,cvec2] = sort(cvec); %sort according to optimal assignments
                        out_vec_1 = cvec2(1:nsamp_slice)>nsamp_template; %logical markings
                        out_vec_2 = cvec(1:nsamp_template)>nsamp_slice;

                        X2 = NaN*ones(nptsd,2);
                        X2(1:nsamp_slice,:) = Xk;
                        X2 = X2(cvec,:);
                        X2b = NaN*ones(nptsd,2);
                        X2b(1:nsamp_slice,:) = X_0;
                        X2b = X2b(cvec,:);
                        Y2 = NaN*ones(nptsd,2);
                        Y2(1:nsamp_template,:) = Y_0;

                        %extract coordinates of non-dummy correspondences and
                        %use them to estimate transformations
                        ind_good = find(~isnan(X2b(1:nsamp_slice,1)));
        
                        X3b = X2b(ind_good, :);
                        Y3 = Y2(ind_good, :);
                    %end %end instert ~ITERATIVE RIGID FLAG
                    
                    
                    
                    
                    %remove incorrect/outlier/impossible Hungarian matches
                    X4b=[]; Y4=[];
                    if(INCORRECT_HUNGARIAN_FLAG)
                        X4b = X2b(ind_good,:); Y4 = Y2(ind_good,:);
                        X4b_1 = X4b(:,1); Y4_1 = Y4(:,1); %x-coord
                        X4b_2 = X4b(:,2); Y4_2 = Y4(:,2); %y-coord
                        %find distances
                        dvec = sqrt((X4b_1-Y4_1).^2 + (X4b_2-Y4_2).^2);
                        epsillon_nonrigidtmp = mean(dvec)+std(dvec);
                        dvec_indx = dvec <= epsillon_nonrigid;


                        X3b = X3b(dvec_indx,:);
                        Y3 = Y3(dvec_indx,:);
                        t2x = t2(dvec_indx,:);
                        tkx = tk(dvec_indx,:);
                        %dvec_vec = dvec_vec(dvec_indx);
                        n_good_hungarian = sum(dvec_indx);
                        run_n_good = run_n_good + n_good_hungarian;
						n_good_hungarian_vec = [n_good_hungarian_vec; n_good_hungarian]; %store # of good correspondences
                        %n_good_vector(nx) = n_good_hungarian; %update n_good_vector, need this otherwise:out of bounds for samples points to pick
                        disp(['DistanceVec NONRIGID mode, Threshold= ' num2str(mean(dvec)+std(dvec)) ', ' num2str(epsillon_nonrigid) ', n_good= ' int2str(n_good_hungarian)])
                    end

                    disp([int2str(n_good_hungarian) ' correspondences (warped X/slice)']);
                    %store to X3bag and Y3bag
                    X3bag = [X3bag; X3b];
                    Y3bag = [Y3bag; Y3];
                    tknbag = [tknbag; tkx];
                    t2nbag = [t2nbag; t2x];
                    %dvecbag = [dvecbag; dvec_vec];
                    INVALID_FLAG = 0;
                
                    %calculate shape context cost
                    [a1,b1] = min(run_costmat, [], 1);
                    [a2,b2] = min(run_costmat, [], 2);
                    sc_cost = sc_cost + max(mean(a1), mean(a2));
                    disp(['total n_good : ' int2str(run_n_good) ' sc_cost: ' num2str(sc_cost)]);
                    
                if(DISPLAY_FLAG && nx==3)
                    fig_index = fig_index + 1;
                    figure(fig_index)
                    
                    % show the correspondences between the
                    % untransformed images for all orientations
                    %plot(X_0(:,1),X_0(:,2),'b+',Y_0(:,1),Y_0(:,2),'ro');
                    hold on
                    plot(X3b(:,1),X3b(:,2),'gs',Y3(:,1),Y3(:,2),'md');
                    plot([X3b(:,1) Y3(:,1)]',[X3b(:,2) Y3(:,2)]','k-')
                    hold off
                    axis('ij')
                    title([int2str(n_good_hungarian) ' correspondences (unwarped X/slice)' ' k=' int2str(k) ' ox=' int2str(nx)])
                    axis([1 N2 1 N1])
                    drawnow
                end
                    
                    
                else
                    INVALID_FLAG = 1;
                end
            end %per n_good_vector, orientation
                
                X3b = X3bag; Y3 = Y3bag; %update matrices to get all orientations
%                 t2N = t2nbag;
                tkN = tknbag;
%                 Yn = Y3bag;
                %update nsamp_template and nsamp_slice to get all orientations
                nsamp_slice = length(X3b(:,1));
                nsamp_template = length(Y3(:,1));

                disp(['kx= ' int2str(kx) ' iter= ' int2str(k) ' nsamp_template: ' int2str(nsamp_template) ' nsamp_slice: '  int2str(nsamp_slice)]);

                if(DISPLAY_FLAG)
                    fig_index = fig_index + 1;
                    figure(fig_index)
                    % show the correspondences between the
                    % untransformed images for all orientations
                    %plot(Xnew(:,1),Xnew(:,2),'b+',Ynew(:,1),Ynew(:,2),'ro');
                    hold on
                    plot(X3b(:,1),X3b(:,2),'gs',Y3(:,1),Y3(:,2),'md');
                    plot([X3b(:,1) Y3(:,1)]',[X3b(:,2) Y3(:,2)]','k-')
                    hold off
                    axis('ij')
                    title([int2str(run_n_good) ' correspondences (unwarped X/slice)' ' k=' int2str(k)])
                    axis([1 N2 1 N1])
                    drawnow
                end


                %evaluate TPS
                if(~REDUCE_DOF_FLAG)
                    %use bookstein
                    disp(['thin plate splines, bookstein.. regularization beta_k=' num2str(beta_k) ' meandist2=' num2str(mean_dist_2) ' lambda_o=' num2str(lambda_o)])
                    disp('affine_start_flag=')
                    affine_start_flag
                    newX3bn = X3b;
                    newY3n = Y3;
                    %newdvecbag = dvecbag;
                    [cx,cy,E] = bookstein(X3b,Y3,beta_k);
                else
                    %use bookstein
                    disp(['reducing degrees of freedom for splines...[slice_info ' num2str(kx) ']'])
                    avgC_slice = mat2cell(zeros(ly,lx),repmat(step,1,ly/step),repmat(step,1,lx/step));
                    avgC_template = mat2cell(zeros(ly,lx),repmat(step,1,ly/step),repmat(step,1,lx/step));
                    avgC_sliceX = mat2cell(zeros(1,lx), 1,repmat(step,1,lx/step));
                    avgC_sliceY = mat2cell(zeros(ly,1), repmat(step,1,ly/step),1);
                    avgC_templateX = avgC_sliceX;
                    avgC_templateY = avgC_sliceY;
                    mx = ceil(X3b./step); %X3b is in the format of [x y], mx=[col,row], found indices
                    my = ceil(Y3./step); %found indices
                    %keep within bounds
                    mx(:,1) = min(mx(:,1),lx/step); %limit x,cols to lx/step
                    my(:,1) = min(my(:,1),lx/step); %limit x,cols to lx/step
                    mx(:,2) = min(mx(:,2),ly/step); %limit y,rows to ly/step
                    my(:,2) = min(my(:,2),ly/step); %limit y,rows to ly/step
                    pollz = mx(:,1)<=0;
                    mx(pollz,1) = 1;
                    pollz = my(:,1)<=0;
                    my(pollz,1) = 1;
                    pollz = mx(:,2)<=0;
                    mx(pollz,2) = 1;
                    pollz = my(:,2)<=0;
                    my(pollz,2) = 1;
					disp('sizes of avgC everything...')
					size(avgC_sliceX)
					size(avgC_sliceY)
					size(avgC_templateX)
					size(avgC_templateY)
					disp('points to indices computation.. ')
                    for px=1:length(mx(:,1))
                        %[X3b(px,:), Y3(px,:)]
						%[mx(px,:) , my(px,:)]
                        bsx = avgC_sliceX{mx(px,1)}; %fetch the block in X-dim for slice
                        bsy = avgC_sliceY{mx(px,2)}; %fetch the block in Y-dim for slice
                        btx = avgC_templateX{my(px,1)}; %fetch the block in X-dim for template
                        bty = avgC_templateY{my(px,2)}; %fetch the block in Y-dim for template
                        bsx(1) = bsx(1) + X3b(px,1); %add the coordinate displacement in that block
                        bsx(2) = bsx(2) + 1; %add the #
                        bsy(1) = bsy(1) + X3b(px,2); %add the coordinate displacement
                        bsy(2) = bsy(2) + 1; %add the #
                        btx(1) = btx(1) + Y3(px,1); %add the coordinate displacement
                        btx(2) = btx(2) + 1; %add the #
                        bty(1) = bty(1) + Y3(px,2); %add the coordinate displacement
                        bty(2) = bty(2) + 1; %add the #
                        %store it all back
                        avgC_sliceX{mx(px,1)} = bsx;
                        avgC_sliceY{mx(px,2)} = bsy;
                        avgC_templateX{my(px,1)} = btx;
                        avgC_templateY{my(px,2)} = bty; 
                        
                    end
                    
                    X3bn = zeros(length(mx(:,1)),2);
                    Y3n = zeros(length(mx(:,1)),2);
                    indx_2 = 0;
                    
                    disp('averaging displacements in blocks to reduce spline numbers..[x_slice, y_slice x_template, y_template]')
                    for px=1:length(mx(:,1))
                        bsx = avgC_sliceX{mx(px,1)}; %fetch the block in X-dim for slice, lx/step max
                        bsy = avgC_sliceY{mx(px,2)}; %fetch the block in Y-dim for slice, ly/step max
                        btx = avgC_templateX{my(px,1)}; %fetch the block in X-dim for template
                        bty = avgC_templateY{my(px,2)}; %fetch the block in Y-dim for template
                        %[mx(px,1) mx(px,2) my(px,1) my(px,2)]
                        tmp_pt_slice = [bsx(1)./bsx(2), bsy(1)./bsy(2)];
                        tmp_pt_template = [btx(1)./btx(2), bty(1)./bty(2)];
                        %disp(['correspondence [' num2str(indx_2) '] computed as: S(' num2str(tmp_pt_slice(1)) ',' num2str(tmp_pt_slice(2)) ') to T(' num2str(tmp_pt_template(1)) ',' num2str(tmp_pt_template(2)) ')']);
                        indx_2 = indx_2 + 1;
                        X3bn(indx_2,:) = tmp_pt_slice;
                        Y3n(indx_2,:) = tmp_pt_template;
                    end
                    disp('averaging displacements in blocks to reduce spline numbers.. Done');
                    %get rid of repetitions for bookstein to work
                    X3bim = X3bn(:,1) + sqrt(-1)*X3bn(:,2);
                    Y3im = Y3n(:,1) + sqrt(-1)*Y3n(:,2);
                    X3bim_gold = X3bim; Y3im_gold = Y3im;
                    indx_2 = 0;
                    X3bg = []; Y3g = []; %store non-repeats
                    rep_vec = [];
                    
                    while(~isempty(X3bim))
                        %select 1st value, it will be deleted afterwards,
                        %hence, 1st value will always be new per iteration
                        pollx = (X3bim==X3bim(1)); %find a value, v_i
                        tmpX3bim = X3bim(pollx); %find repeated values, v_i
                        tmpY3im = Y3im(pollx); %find this v_i in Y vector
                        %check which Y values, y_i, are the same for v_i
                        [~,px2,~] = unique(tmpY3im); %find non-repeats n y_i
                        uniqY3 = tmpY3im(px2); %found unique values
                        X3bg = [X3bg; repmat(X3bim(1),length(uniqY3),1)]; %store
                        Y3g = [Y3g; uniqY3]; %store
                        
                        rep_vec = [rep_vec; repmat(sum(pollx)-length(uniqY3),length(uniqY3),1)];
                        
                        disp(['Removing repetitive point correspondences, ' '[slice_info ' num2str(kx) '] selection: (' num2str(real(X3bim(1))) ',' num2str(imag(X3bim(1))) ') to (' num2str(real(Y3im(1))) ',' num2str(imag(Y3im(1))) ') found repeats: ' num2str(sum(pollx)-length(uniqY3))])
                        
                        %get rid of values in X3bim and Y3im
                        X3bim(pollx) = []; %get rid of all considered
                        Y3im(pollx) = []; %get rid of all considered
                    end
                    
                    rep_vec = [];
                    for px=1:length(X3bg)
                        hx = X3bg(px);
                        hy = Y3g(px);
                        pollx = (X3bim_gold==hx); %find all repetitions in original vector
                        polly = (Y3im_gold==hy); %find all repetitions in original vector template
                        poll_int = pollx & polly; %find all the logical-1 AND them hx,hy are paired
                        disp(['FREQUENCIES, [slice_info ' num2str(kx) '] selection: (' num2str(real(hx)) ',' num2str(imag(hx)) ') to template (' num2str(real(hy)) ',' num2str(imag(hy)) ') found repeats: ' num2str(sum(poll_int))])
                        rep_vec = [rep_vec; sum(poll_int)]; %store corresponding repeat values for X3bg Y3g
                    end

                    X3bn = [real(X3bg), imag(X3bg)];
                    Y3n = [real(Y3g), imag(Y3g)];
                    X3bn_gold = X3bn; Y3n_gold = Y3n;
                    newX3bn = [];
                    newY3n = [];
                    

                    disp(' ')
                    disp('finding the most popular correspondences and removing extraneous correspondences.. ');
                    mx = ceil(X3bn./step); %X3b is in the format of [x y], mx=[col,row], found indices
                    my = ceil(Y3n./step); %found indices
                    Xtot = [X3bn, Y3n];
                    Xtot = sum(Xtot,2); %sum, same length as X3bn
                    
                    %sort rep_vec and Xtot
                    [~,repinx] = sort(rep_vec,'descend');
                    Xtot = Xtot(repinx);
                    rep_vec = rep_vec(repinx);
                    X3bn_s = X3bg(repinx);
                    Y3n_s = Y3g(repinx);
                    X3bn_sorted = [real(X3bn_s), imag(X3bn_s)];
                    Y3n_sorted = [real(Y3n_s), imag(Y3n_s)];

                    for px=1:length(Xtot(:,1))
                        %consider first sum
                        hx = Xtot(px); %select one element 
                        if(~isnan(hx))
                            disp(['[slice_info ' num2str(kx) '] considering...' num2str(hx) ' at ' num2str(px) ' slice (' num2str(X3bn_sorted(px,1)) ',' num2str(X3bn_sorted(px,2)) ') to template (' num2str(Y3n_sorted(px,1)) ',' num2str(Y3n_sorted(px,2)) ')'])
                            %[X3bn(px,:), Y3n(px,:)]
                            %[mx(px,:), my(px,:)]
                            Xtot_sub = Xtot - repmat(hx,size(Xtot)); %length is X3bn
                            %find the closes distances, should be off by 1
                            Xtot_vec = abs(Xtot_sub) <= step; %found all similar elements, length is X3bn
                            relevant_vec = rep_vec(Xtot_vec); %found corresponding repetitions/votes, length is diff
                            %now match which one has the most repetitions/votes
                            [cdx,cdxi] = max(relevant_vec); %found max

                            %find the first index of cdxi=5, in Xtot_vec (logical)
                            myind = 0;
                            superindex = 0;
                            for py=1:length(Xtot_vec)
                                
                                if(Xtot_vec(py))
                                    %found a 1 increment myind
                                    myind = myind + 1;
                                    if(myind == cdxi) %found the real index
                                        superindex = py;
                                    end
                                end
                            end
                            
                            %test for uniqueness, bug in the program i
                            if(~isempty(newX3bn))
                                
                                fd = X3bn_sorted(superindex,:);
                                diff_fd = (newX3bn - repmat(fd, length(newX3bn(:,1)),[])); %subtract the coordinates
                                diff_fd = sum(diff_fd,2); %sum it, look for 0, this means it already exists
                                diff_fd = (diff_fd==0); %logical index, find 0
                                diff_fd = sum(diff_fd); %should be 0 if it is new to the list, o.w. it should be 1
                                if(diff_fd == 0 && cdx>max_thresh_dof) %no such exists, add it!
                                    %store values  unique
                                    newX3bn = [newX3bn; X3bn_sorted(superindex,:)];
                                    newY3n = [newY3n; Y3n_sorted(superindex,:)];
                                    disp(['[slice_info ' num2str(kx) '] found a good/unique correspondence based on repetition for ' num2str(hx) ',max_thresh_dof=' num2str(max_thresh_dof) ' at superindex=' num2str(superindex) ' max=' num2str(cdx) ',max_index=' num2str(cdxi) ' value is (' num2str(X3bn(superindex,1)) ',' num2str(X3bn(superindex,2)) ') to template: (' num2str(Y3n(superindex,1)) ',' num2str(Y3n(superindex,2)) ')' ]);
                                end
                            else
                                %storing the most-relevant/repeated element
                                %first iteration
                                newX3bn = [newX3bn; X3bn_sorted(superindex,:)];
                                newY3n = [newY3n; Y3n_sorted(superindex,:)];
                                disp(['[slice_info ' num2str(kx) '] found a good correspondence based on repetition for ' num2str(hx) ' at superindex=' num2str(superindex) ' max=' num2str(cdx) ',max_index=' num2str(cdxi) ' value is (' num2str(X3bn(superindex,1)) ',' num2str(X3bn(superindex,2)) ') to template: (' num2str(Y3n(superindex,1)) ',' num2str(Y3n(superindex,2)) ')' ]);
                            end
                            
                            
                            disp('---')
                            %deleting everything
                            Xtot(Xtot_vec) = NaN;
                            %X3bn(Xtot_vec) = [];
                            %Y3n(Xtot_vec) = [];
                            %mx(Xtot_vec) = [];
                            %my(Xtot_vec) = [];
                            %rep_vec(Xtot_vec) = NaN;
                        end
                    end
                    
                    run_n_good = length(newX3bn(:,1));
                    disp(['going to match the following.. using bookstein, (slice to template) [slice_info ' int2str(kx)])
                    [newX3bn , newY3n]
                    disp(['thin plate splines, bookstein.. regularization=' num2str(beta_k) ' meandist2=' num2str(mean_dist_2) ' lambda_o=' num2str(lambda_o) ' numSplines=' num2str(run_n_good)])
                    %disp('affine_start_flag=')
                    affine_start_flag
                    [cx,cy,E] = bookstein(newX3bn,newY3n,beta_k);
                end
                
%                 disp('bookstein result..')
%                 [cx cy]
                disp('AFFINE COST computation')
                A = [cx(run_n_good+2:run_n_good+3,:) cy(run_n_good+2:run_n_good+3,:)]
                s = svd(A);
                aff_cost = log(s(1)/s(2)); %affine cost


                fx_aff = cx(run_n_good+1:run_n_good+3)'*[ones(1,nsamp_slice_0); X'];
                d2 = max(dist2(newX3bn,X),0);
                U = d2.*log(d2 + eps);
                fx_wrp = cx(1:run_n_good)'*U;
                fx = fx_aff + fx_wrp;
                fy_aff = cy(run_n_good+1:run_n_good+3)'*[ones(1,nsamp_slice_0); X'];
                fy_wrp = cy(1:run_n_good)'*U;
                fy = fy_aff + fy_wrp;
                Z = [fx; fy]'; %COORDINATES?

                %apply warp to tangent vectors to get new angles
                Xtan = X + tan_eps*[cos(t1) sin(t1)];
                fx_aff = cx(run_n_good+1:run_n_good+3)'*[ones(1,nsamp_slice_0); Xtan'];
                d2 = max(dist2(newX3bn,Xtan),0);
                U = d2.*log(d2+eps);
                fx_wrp = cx(1:run_n_good)'*U;
                fx = fx_aff + fx_wrp;
                fy_aff = cy(run_n_good+1:run_n_good+3)'*[ones(1,nsamp_slice_0); Xtan'];
                fy_wrp = cy(1:run_n_good)'*U;
                fy = fy_aff + fy_wrp;
                Ztan = [fx; fy]'; %angle  warps
                tk = atan2(Ztan(:,2)-Z(:,2),Ztan(:,1)-Z(:,1));
                
                tkN = tk; %update tk
                Xn = Z; %update Xk for next iteration
                %newX3bn = Xn; %CAUSES ERRORS
                
                if(DISPLAY_FLAG)
                    fig_index = fig_index + 1;
                    figure(fig_index)
                    plot(Z(:,1),Z(:,2),'b+',Yn(:,1),Yn(:,2),'ro');
                    axis('ij')
                    title(['k=' int2str(k) ', \lambda_o=' num2str(lambda_o) ', I_f=' num2str(E) ', aff.cost=' num2str(aff_cost) ', SC cost=' num2str(sc_cost)])
                    axis([1 N2 1 N1])
                    % show warped coordinate grid
                    fx_aff = cx(run_n_good+1:run_n_good+3)'*[ones(1,M); x'; y'];
                    d2 = dist2(newX3bn,[x y]);
                    fx_wrp = cx(1:run_n_good)'*(d2.*log(d2+eps));
                    fx = fx_aff + fx_wrp;
                    fy_aff = cy(run_n_good+1:run_n_good+3)'*[ones(1,M); x'; y'];
                    fy_wrp = cy(1:run_n_good)'*(d2.*log(d2+eps));
                    fy = fy_aff + fy_wrp;
                    hold on
                    plot(fx,fy,'k.','markersize',4)
                    hold off
                    drawnow
                end

                %update iteration number

                if (k == n_iter)
                    NONRIGID_FLAG = 0; %end loop
                else
                    k = k+1;
                end
                disp(['starting iter=' int2str(k)])
            
        end %n_iter for non-rigid
        
     end
    
     %STORE WARPS after non-rigid loop ended
     [x,y] = meshgrid(1:N2,1:N1);
     x = x(:);y = y(:);M = length(x);
     if NONRIGID_FLAG_RUN
         fx_aff = cx(run_n_good+1:run_n_good+3)'*[ones(1,M); x'; y'];
         d2 = dist2(newX3bn,[x y]);
         fx_wrp = cx(1:run_n_good)'*(d2.*log(d2+eps));
         fx = fx_aff + fx_wrp;
         fy_aff = cy(run_n_good+1:run_n_good+3)'*[ones(1,M); x'; y'];
         fy_wrp = cy(1:run_n_good)'*(d2.*log(d2+eps));
         fy = fy_aff + fy_wrp;
         %compensation for rigid-body
         X6b = [fx;fy];
         RX = HX(1:2, 1:2);
         tX = HX(:,3); tX = tX(1:2);
         RX
         if(~REDUCE_DOF_FLAG && ~ITERATIVE_RIGID_FLAG) %IR=0 NR=1 DOF=0
             RX = RX;
             tX = tX;
             disp('weird case.. RX tX NONRIGID')
         else %everything else
             RX = RX; 
             tX = tX;
             disp('switch case.. RX -tX NONRIGID, no RXtranspose')
         end
         disp('storing warps..NONRIGID')
         RX
         tX
         X6b = RX*X6b + repmat(tX,1,length(X6b(1,:))); %initially



         fx = X6b(1,:);
         fy = X6b(2,:);
     else
         %ONLY RIGID
         X6b = [x,y]';
         RX = HX(1:2, 1:2);
         tX = HX(:,3); tX = tX(1:2);
         RX
         if(~ITERATIVE_RIGID_FLAG) %IR=0 NR=1 DOF=0
             RX = RX;
             tX = tX;
             disp('weird case.. RX tX RIGID')
         else %everything else
             RX = RX; 
             tX = tX;
             disp('switch case.. RX -tX RIGID no RXtranspose - didnt happen')
         end
         disp('storing warps..RIGID')
         RX
         tX
         X6b = RX*X6b + repmat(tX,1,length(X6b(1,:))); %initially



         fx = X6b(1,:);
         fy = X6b(2,:);
     end
     regy = newY3n'; regx = newX3bn';
     fre_squaredx = (1/length(regy(1,:)))*sum(abs(regx-regy).^2,2)
     fre_sqx_mag = sqrt(fre_squaredx(1).^2 + fre_squaredx(2).^2);
     disp(['final FRE.. [slice_info ' num2str(kx) '] FRE-sq: ' num2str(fre_sqx_mag)]); 
     fre_square_mag_vec(kx) = fre_sqx_mag;
     
     maxVesslStk(kx).deform.fx = reshape(fx,N1,N2); %works as deformTotal_fx in previous versions
     maxVesslStk(kx).deform.fy = reshape(fy,N1,N2);
     maxVesslStk(kx).deform.correspondencesTemplate = newX3bn;
     maxVesslStk(kx).deform.correspondencesSlice = newY3n;
     maxVesslStk(kx).deform.rigid_H = HX;
     %maxVesslStk(kx).deform.flag = 0;
     maxVesslStk(kx).deform.n_good_per_orientation = n_good_hungarian_vec;

    %sbvec = newY3n-newX3bn;
    %rms.NonRigidcorrRMS(kx).data = sbvec; %for rms
    rms.NonRigidcorrRMS(kx).Xbag = newX3bn; %Slice assignments
    rms.NonRigidcorrRMS(kx).Ybag = newY3n; %template assignments 
    
     fig_index = fig_index + 10;
     
     disp(['...COMPLETED: warp of slice ' int2str(kx) ' to the template ' ])
     toc
     disp('_________________________________');
     disp(['flags are .. ITERATIVE_RIGID_FLAG=' num2str(ITERATIVE_RIGID_FLAG) ' NONRIGID_FLAG_RUN=' num2str(NONRIGID_FLAG_RUN) ' REDUCE_DOF_FLAG=' num2str(REDUCE_DOF_FLAG) ])
     RIGID_RUN_FLAG = RIGID_RUN_FLAG_GOLD;
end
indx_2 = (1:lz)';
disp('fre-squared-magnitude-vector for all slices')
[indx_2, fre_square_mag_vec]
%WRITE
if(WRITE_FLAG)
    num_orients = maxTEMPLATEVesslStk.num_orientations; 
    outfn1 = strcat(filepath1, filename1);
    %outfn1 = strcat(outfn1, '_RXon'); %AffineFlag=0 for DOF, and Max_Thresh_DOF = 8
    thresh_low = 0.005; %hysteresis
    thresh_high = 0.1; %hysteresis
    sz = size(maxVesslStk(1).vesselness);
    radius = 1.5; %non maxima threshold
    N1 = ly; N2 = lx;
    [x,y] = meshgrid(1:N2,1:N1);
    x = x(:);y = y(:);M = length(x);
    outfname1 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl.tif');
    outfname2 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl-stk.tif');
    outfname3 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMS.tif');
    outfname4 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMShyst.tif');
    outfname5 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMSavg.tif');
    outfname6 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMShystAVG.tif');
	outfname7 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMS_allsliceAVG.tif');
	outfname8 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_NMShyst_allsliceAVG.tif');
    outfname9 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid), '_B',num2str(beta_init),'_a',num2str(r),'_vessl_DefGrid.tif');
    outfname10 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r),'_vessl_DefGrid_MarkedPts.tif');
    outfname11 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r),'_vessl_DefGrid_DefPts.tif');
    outfname12 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r),'_vessl-stk-orig.tif');
    outfname13 = strcat(outfn1,'_XDeform_','smpl',num2str(number_samples),'_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r),'_rbc-stk-orig.tif');
    writeimgsum = zeros(sz);
    bwimgsum = zeros(sz);
	vimgsum = zeros(sz);
    outfname1
	for indx_corr=1:lz
		%fetch index of motion corrected frame
		kx = ncc_ranges_all(indx_corr); %0 for not motion corrected, 1 for motion corrected
		if(kx~=0) %motion correction was performed
			disp(['Writing Correction of slice number.. ', int2str(kx)]);
			xdimg = zeros(sz);
			stk_img = double(stk_vessl(kx).data);
			stk_img_vessl_orig = double(stk_vessl_orig(kx).data);
			stk_img_rbc_orig = double(stk_rbc_orig(kx).data);
			vessl_img = maxVesslStk(kx).vesselness;
			Xbg = maxVesslStk(kx).deform.correspondencesTemplate;
			Ybg = maxVesslStk(kx).deform.correspondencesSlice;
			xbgy = ceil(Xbg(:,2)); xbgx = ceil(Xbg(:,1));
			xbg_inx = xbgy<=0; xbgy(xbg_inx) = 1;
			xbg_inx = xbgx<=0; xbgx(xbg_inx) = 1;
			xbg_inx = xbgy>ly; xbgy(xbg_inx) = ly;
			xbg_inx = xbgx>lx; xbgx(xbg_inx) = lx;
			xdimg(xbgy,xbgx) = 64; %mark slice points
			xbgy = ceil(Ybg(:,2)); xbgx = ceil(Ybg(:,1));
			xbg_inx = xbgy<=0; xbgy(xbg_inx) = 1;
			xbg_inx = xbgx<=0; xbgx(xbg_inx) = 1;
			xbg_inx = xbgy>ly; xbgy(xbg_inx) = ly;
			xbg_inx = xbgx>lx; xbgx(xbg_inx) = lx;
			xdimg(xbgy,xbgx) = 192; %mark template points
			fx =  maxVesslStk(kx).deform.fx; %total deformation in x
			fy = maxVesslStk(kx).deform.fy; %total deformation in y
			t_img = griddata(fx,fy,vessl_img,reshape(x,N1,N2),reshape(y,N1,N2));   
			v_img = t_img;
            maxVesslStk(kx).deformvesselness = v_img; %store deformed/corrected vesselness values
			vimgsum = vimgsum + v_img; %add all
			t_imgstk = griddata(fx,fy,stk_img,reshape(x,N1,N2),reshape(y,N1,N2)); %stk_vessl (preprocessed - deconv/CLAHE)
			t_imgstk_vessl_orig = griddata(fx,fy,stk_img_vessl_orig,reshape(x,N1,N2),reshape(y,N1,N2)); %stk_vessl (original)
			t_imgstk_rbc_orig = griddata(fx,fy,stk_img_rbc_orig,reshape(x,N1,N2),reshape(y,N1,N2)); %stk_vessl (original)
			dg_img = griddata(fx,fy,def_grid,reshape(x,N1,N2),reshape(y,N1,N2)); %deformation grid
			ydimg = griddata(fx,fy,xdimg,reshape(x,N1,N2),reshape(y,N1,N2)); %deformation grid with points on it
			t_img = 2.*t_img;
			t_img = uint8(255*(t_img./max(t_img(:)))); 
			t_imgstk = uint8(255*(t_imgstk./max(t_imgstk(:)))); %preprocessed stk
			t_imgstk_vessl_orig = uint8(255*(t_imgstk_vessl_orig./max(t_imgstk_vessl_orig(:)))); %original vessl stk
			t_imgstk_rbc_orig = uint8(255*(t_imgstk_rbc_orig./max(t_imgstk_rbc_orig(:)))); %original rbc stk
			dg_img = uint8(255*(dg_img./max(dg_img(:)))); 
			ydimg = uint8(255*(ydimg./max(ydimg(:)))); 
			xdimg = uint8(255*(xdimg./max(xdimg(:)))); 
			imwrite(t_img, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(t_imgstk, outfname2 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write preprocessed
			imwrite(t_imgstk_vessl_orig, outfname12 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write original vessl
			imwrite(t_imgstk_rbc_orig, outfname13 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write rbc vessl
			imwrite(dg_img, outfname9 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(xdimg, outfname10 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(ydimg, outfname11 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			%nonmaxima suppression and hysteresis
			[im,loc]=nonmaxsup(v_img, 90*ones(sz), radius);
			bw = hysthresh(im,thresh_high, thresh_low); %hysteresis
			maxVesslStk(kx).deform.nonmaxsupImage = im;
			maxVesslStk(kx).deform.hysteresis = bw;
			t_img = 2.*im;
			writeimgsum = writeimgsum+t_img; %for averaging NMS
			imwrite(t_img, outfname3 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			t_img = 2.*bw;
			bwimgsum = bwimgsum+t_img; %for averaging NMS_hysteresis
			imwrite(t_img, outfname4 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			kx
		else
			%motion not corrected, still need to write files
			kx = indx_corr; %fetch index of not motion corrected frame, default
            disp(['Writing no-Correction of slice number.. ', int2str(kx)]);
			t_imgstk = double(stk_vessl(kx).data);
			t_imgstk_vessl_orig = double(stk_vessl_orig(kx).data);
			t_imgstk_rbc_orig = double(stk_rbc_orig(kx).data);
			vessl_img = maxVesslStk(kx).vesselness;
			t_img = vessl_img;
			v_img = t_img;
            maxVesslStk(kx).deformvesselness = v_img;
			vimgsum = vimgsum + v_img; %add all
			t_img = 2.*t_img;
			t_img = uint8(255*(t_img./max(t_img(:)))); 
			t_imgstk = uint8(255*(t_imgstk./max(t_imgstk(:)))); %preprocessed stk
			t_imgstk_vessl_orig = uint8(255*(t_imgstk_vessl_orig./max(t_imgstk_vessl_orig(:)))); %original vessl stk
			t_imgstk_rbc_orig = uint8(255*(t_imgstk_rbc_orig./max(t_imgstk_rbc_orig(:)))); %original rbc stk
			imwrite(t_img, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(t_imgstk, outfname2 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write preprocessed
			imwrite(t_imgstk_vessl_orig, outfname12 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write original vessl
			imwrite(t_imgstk_rbc_orig, outfname13 ,'tif', 'Compression', 'none', 'WriteMode', 'append'); %write rbc vessl
			dg_img = def_grid;
			dg_img = uint8(255*(dg_img./max(dg_img(:)))); 
			xdimg = zeros(sz);
			ydimg = uint8(255*(xdimg./max(xdimg(:)))); 
			xdimg = uint8(255*(xdimg./max(xdimg(:))));
			imwrite(dg_img, outfname9 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(xdimg, outfname10 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			imwrite(ydimg, outfname11 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			%nonmaxima suppression and hysteresis
			[im,loc]=nonmaxsup(v_img, 90*ones(sz), radius);
			bw = hysthresh(im,thresh_high, thresh_low); %hysteresis
			t_img = 2.*im;
			writeimgsum = writeimgsum+t_img; %for averaging NMS
			imwrite(t_img, outfname3 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			t_img = 2.*bw;
			bwimgsum = bwimgsum+t_img; %for averaging NMS_hysteresis
			imwrite(t_img, outfname4 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
			kx
		end
    end
    vimgsum = vimgsum./lz;
    writeimgsum = writeimgsum./lz;
    bwimgsum = bwimgsum./lz;
    imwrite(writeimgsum, outfname5 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    imwrite(bwimgsum, outfname6 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
	[im,loc]=nonmaxsup(vimgsum, 90*ones(sz), radius);
	bw = hysthresh(im,thresh_high, thresh_low); %hysteresis
	maxVesslStk(1).deform.nonmaxsupImageAVG = im;
	maxVesslStk(1).deform.hysteresisAVG = bw;
	t_img = 2.*im;
	imwrite(t_img, outfname7 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
	t_img = 2.*bw;
	imwrite(t_img, outfname8 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end


if(SAVE_MAXVESSLSTK)
    disp('saving maxvesslstk..')
    savefilename = strcat(filename1,'_maxVessl_DEFORM','_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r));
    savefilename
    savefilenamerms = strcat(filename1,'_maxVessl_RMS','_r',num2str(epsillon_rigid),'_nr',num2str(epsillon_nonrigid),'_B',num2str(beta_init),'_a',num2str(r));
    savefilenamerms
    save(savefilenamerms,'rms','epsillon_nonrigid','epsillon_rigid','number_samples','-v7.3');
    save(savefilename,'maxVesslStk','epsillon_nonrigid','epsillon_rigid','number_samples','n_iter','r','beta_init','-v7.3');
    disp('DONE');
else
    disp('Not saving maxvesslstk.. takes too long');
    disp('DONE');
end



