% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%multiple orientations for vessels, maxVesslStk
% POST non-maxima-suppression and hysteresis
%compute orientation based on the eigenvector2 (normal direction) to 0,0,0

function maxVesslStk = multipleOrientations(filepath1,filename1,maxVesslStk)

WRITE_FLAG = 0; %0: don't output files

lz = length(maxVesslStk); %length of maxVesslStk
sz = size(maxVesslStk(1).vesselness);
num_divisions = 8;
step_angle = 2*pi/(num_divisions);
lz = length(maxVesslStk); %length of maxVesslStk
ly = length(maxVesslStk(1).vesselness(:,1)); %num rows
lx = length(maxVesslStk(1).vesselness(1,:)); %num cols
%output to an image
% filename1 = '8t_ch2_451-525_sm_deconv_15CLAHE'; %deconvolved+CLAHE image VESSEL
% filepath1 = strcat(pwd,'\..\images\24_February_2011\results\8t\vesselness\');
outfn1 = strcat(filepath1, filename1);
outfname1 = strcat(outfn1, '_NMShyst','_orient',num2str(num_divisions),'_vessl.tif');

tic
for k=1:lz %slice number
    vc1 = maxVesslStk(k).eigVec2(1).vector;
    vc2 = maxVesslStk(k).eigVec2(2).vector;
    vcc1 = zeros(size(vc1));
    vcc2 = zeros(size(vc2));
    hystmat = maxVesslStk(k).hysteresis; %non-maxima-supp, hysteresis
    vcc1(hystmat) = vc1(hystmat);
    vcc2(hystmat) = vc2(hystmat);
    angles = atan2(vcc2,vcc1) + pi/2; %all angles
%     indx = angles<0;
%     angles(indx) = 2*pi+angles(indx); %between [0,2pi];
    orient_index =  floor(angles/step_angle); %compute orientation index
    mnx = mode(orient_index(:)); %background pixel
    minx = abs(min(orient_index(:)))+1;
    inx = orient_index ~= mnx; %get non-background pixels
    orients = zeros(size(orient_index));
    orients(inx) = orient_index(inx) + minx;
    maxVesslStk(k).angles = angles;
    maxVesslStk(k).orientations = orients;
    maxVesslStk(k).backgroundOrientationValue = mnx;
%     mnx
end
maxVesslStk(1).num_orientations = num_divisions;
toc

if (WRITE_FLAG)
    for k=1:lz
        writeimg = maxVesslStk(k).orientations;
        writeimg = uint8(255*(writeimg./max(writeimg(:)))); %make into 8bit
        imwrite(writeimg, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    end
end

%%AVERAGE_HYSTERESIS based ORIENTATION %%
%for AVERAGE orientation based on AVERAGE_hysteresis(bwimgsum:NMS_driver)
%bwavg: hysteresis computed on AVERAGE_VESSLNESS
%for average orientation output for average hysteresis

%SELECT one
% bwmatindx = bwimgsum>0; %average hysteresis as logical value, AVERAGE_HYST
bwmatindx = maxVesslStk(1).NMS_HYST_AVG>0; %hysteresis on AVG_VESSLNESS

svc1 = zeros(size(vc1)); svcc1 = svc1;
svc2 = zeros(size(vc2)); svcc2 = svc2;
for k=1:lz
    %find average eigenvector2
    svc1 = svc1 + maxVesslStk(k).eigVec2(1).vector;
    svc2 = svc2 + maxVesslStk(k).eigVec2(2).vector;
end
svc1 = svc1./lz; %average
svc2 = svc2./lz; %average
svcc1(bwmatindx) = svc1(bwmatindx);
svcc2(bwmatindx) = svc2(bwmatindx);
angles = atan2(svcc2,svcc1)+ pi/2;
% indx = angles<0;
% angles(indx) = 2*pi+angles(indx);
orient_avg =  floor(angles/step_angle); %compute orientation index
mnx = mode(orient_avg(:)); %background pixel value, 4
minx = abs(min(orient_avg(:)))+1; %minimum orientation, 2+offset
inx = orient_avg ~= mnx; %get non-background pixels, !=4
orientsaverage = zeros(size(orient_avg));
orientsaverage(inx) = orient_avg(inx) + minx; %non-bg_pix + |min|
orientsaverage = orientsaverage + min(orientsaverage(:)); 
maxVesslStk(1).ORIENTATION_AVG = orientsaverage;
maxVesslStk(1).backgroundOrientationValueAVG = mnx;
maxVesslStk(1).ANGLES_AVG = angles;
% disp('!!!!'); mnx
if(WRITE_FLAG)
    outfname2 = strcat(outfn1, '_NMShyst','_orient',num2str(num_divisions),'_vesslAVERAGE.tif');
%     outfname2 = strcat(outfn1, '_NMShystAVERAGE','_orient',num2str(num_divisions),'_vessl.tif');
    orient_avg = uint8(255*(orientsaverage./max(orientsaverage(:)))); %make into 8bit
    imwrite(orient_avg, outfname2 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end
