% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%NON MAXIMA SUPPRESSION DRIVER [NMS]
%load maxVesslStk as input
%input: maxVesslStk.vesselness, orientation are all 90degrees
function maxVesslStk = nonmaxsup_driver(filepath1,filename1,maxVesslStk)

WRITE_FLAG = 0; %1: output files

lz = length(maxVesslStk); %length of maxVesslStk
ly = length(maxVesslStk(1).vesselness(:,1)); %num rows
lx = length(maxVesslStk(1).vesselness(1,:)); %num cols
sz = size(maxVesslStk(1).vesselness);
radius = 1.5;
vesslimgsum = zeros(sz);
thresh_low = 0.005; %hysteresis
thresh_high = 0.1; %hysteresis
tic
for k=1:lz
    [im,loc]=nonmaxsup(maxVesslStk(k).vesselness, 90*ones(sz), radius);
    bw = hysthresh(im,thresh_high, thresh_low); %hysteresis
    vesslimgsum = vesslimgsum + 2.*maxVesslStk(k).vesselness;
    maxVesslStk(k).nonmaxsupImage = im;
    maxVesslStk(k).edgeLocation = loc; %(row)+j(column)
    maxVesslStk(k).hysteresis = bw;
    k
end
%average vesselness
vesslimgsum = vesslimgsum ./ lz;
[imavg, locavg] = nonmaxsup(vesslimgsum, 90*ones(sz), radius);
bwavg = hysthresh(imavg,thresh_high*2, thresh_low*2); %a factor of 2 was used to compute the average
imavg = imavg.*2;
bwavg = bwavg.*2;
maxVesslStk(1).NMS_HYST_AVG = bwavg;
toc

if(WRITE_FLAG)
%write output to a TIF file
% filename1 = '8t_ch2_451-525_sm_deconv_15CLAHE'; %deconvolved+CLAHE image VESSEL
% filepath1 = strcat(pwd,'\..\images\24_February_2011\results\8t\vesselness\');
outfn1 = strcat(filepath1, filename1);
outfname1 = strcat(outfn1, '_sigMAX','_del15','_NMS_vessl.tif');
outfname5 = strcat(outfn1, '_sigMAX','_del15','_2XNMS_vessl.tif');
outfname6 = strcat(outfn1, '_sigMAX','_del15','_avg2XNMS_vessl.tif');
outfname7 = strcat(outfn1, '_2XsigMAX','_del15','_2XNMS_vesslAVERAGE.tif');
outfname8 = strcat(outfn1, '_sigMAX','_del15','_2XNMS_2XHys_vessl.tif');
outfname9 = strcat(outfn1, '_sigMAX','_del15','_avg2XNMS_avg2XHys_vessl.tif');
outfname10 = strcat(outfn1, '_2XsigMAX','_del15','_2XNMS_2XHys_vesslAVERAGE.tif');
writeimgsum = zeros(sz);
bwimgsum = zeros(sz);
for k=1:lz
    writeimg = maxVesslStk(k).nonmaxsupImage;
    bwimgwrite = maxVesslStk(k).hysteresis;
    imwrite(writeimg, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    writeimg = 2.*writeimg; %2X MAX nonmaxsup
    bwimgwrite = 2.*bwimgwrite;
    imwrite(writeimg, outfname5 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    imwrite(bwimgwrite, outfname8 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    writeimgsum = writeimgsum + writeimg; %create sum of nonmaxsup
    bwimgsum = bwimgsum + bwimgwrite;
end
writeimgsum = writeimgsum./lz; %create average 2X MAX NMS
bwimgsum = bwimgsum./lz;
imwrite(writeimgsum, outfname6 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
imwrite(bwimgsum, outfname9 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
imwrite(imavg, outfname7 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
imwrite(bwavg, outfname10 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end