% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%manipulate a vessel stack to perform transient Ca2+ images
%manipulate the brightness (increase) of a fraction of an imaged blood vessel in a subset of raw data time series
%input: Vesselness Enhanced image stack, uncorrected for motion

path_images = 'C:\Users\kumara4\ankur\Piston\code';
filepath_vessl=strcat(path_images,'/images/24_February_2011/results/3t/vesselness/');
%fname1 = strcat(filepath_vessl, '3t_ch2_751-825_sm_deconv_15CLAHE_sigMAX_del15_vessl'); 
fname1 = strcat(filepath_vessl, '3t_ch2_751-825');
fname1 = strcat(fname1, '.tif');
outfname1 = strcat(filepath_vessl, '3tTrans_ch2_751-825');
outfname1 = strcat(outfname1, '.tif');
%set up
stk_vessl = tiffread29(fname1); %read the stk
width = stk_vessl(1,1).width; %col
height = stk_vessl(1,1).height; %row
lz = length(stk_vessl);
maxVesslStk = [];
for k=1:lz
    ch1_data = double(stk_vessl(k).data); %vessels, from motionCorr_driver
    maxVesslStk(k).vesselness = ch1_data;
end

%compute transients
threshold = 35;
roisize_c = 55;
roisize_r = 45;
offset_c = 10;
offset_r = 10;
scale = 2;
r = 15; c = 25;
rx = r + roisize_r; cx = c + roisize_c;
for k=1:lz
    if(mod(k,2)==1)
        dat = maxVesslStk(k).vesselness;
        datout = dat; %output

        roi = dat(r:rx, c:cx); %get roi
        %manipulate/increase brightness
        roi_indx = roi>threshold; %vesselness
        roi(roi_indx) = roi(roi_indx).*scale;
        datout(r:rx, c:cx) = roi; %output

        r = r + offset_r; c = c + offset_c;
        rx = r + roisize_r; cx = c + roisize_c;
        %stay within bounds
        if(rx>height)
            rx = height;
        end
        if(cx>width)
            cx = width;
        end
        %reset
        if(r>height || c>width)
            r = 25;
            rx = r + roisize_r;
            c = 100;
            cx = c + roisize_c;
        end
        [r rx c cx]
    end
    maxVesslStk(k).vesselnessTransient = datout;
    k
end

%write output out
for k=1:lz
    img = maxVesslStk(k).vesselnessTransient;
    %writeimg = uint8(255*(img./max(img(:))));
    writeimg = uint8(img);
    imwrite(writeimg, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end



